/**
 * Controllers.java
 *
 * Emulates SMS/GG Controllers
 *
 * @author Copyright (C) 2002-2003 Chris White
 * @version 18th January 2003
 * @see "JavaGear Final Project Report"
 */

  /*
    This file is part of JavaGear.

    JavaGear is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    JavaGear is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with JavaGear; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import java.awt.event.*;

public final class Controllers
	implements KeyListener
{
	private Setup setup;

	private int controller1;				// Controller 1
	private int controller2;				// Controller 2
	private int ggstart;					// Game Gear Start Button
	private boolean reset_button;			// SMS Reset Button

	// Default Key Mappings
	private int up = KeyEvent.VK_UP;
	private int down = KeyEvent.VK_DOWN;
	private int left = KeyEvent.VK_LEFT;
	private int right = KeyEvent.VK_RIGHT;
	private int fire1 = KeyEvent.VK_Z;
	private int fire2 = KeyEvent.VK_X;
	private int start = KeyEvent.VK_ENTER;

	/**
	 *	Controllers Constructor.
	 *
     */

	public Controllers(Setup s)
	{
		this.setup = s;
		// Default 0xFF = No Keys Pressed
		controller1 = 0xFF;
		controller2 = 0xFF;
		ggstart = 0xFF;
		reset_button = false;
		setup.pause_button = false;
	}

	/**
	 *	Key Pressed
	 *
	 *  @param evt	Key Code
     */

	public void keyPressed(KeyEvent evt)
	{
		int keyCode = evt.getKeyCode();
		
		if (keyCode == up) 		controller1 &= ~0x01;	// Up
		else if (keyCode == down)	controller1 &= ~0x02;	// Down
		else if (keyCode == left)	controller1 &= ~0x04;	// Left
		else if (keyCode == right)	controller1 &= ~0x08;	// Right
		else if (keyCode == fire1)	controller1 &= ~0x10;	// Fire 1
		else if (keyCode == fire2)	controller1 &= ~0x20;	// Fire 2
		else if (keyCode == start)
		{
			if (setup.is_sms)
				setup.pause_button = true;	// Pause
			else
				ggstart	&= ~0x80;			// Start
		}
	}

	/**
	 *	Key Released
	 *
	 *  @param evt	Key Code
     */

	public void keyReleased(KeyEvent evt)
	{
		int keyCode = evt.getKeyCode();

		if (keyCode == up) 		controller1 |= 0x01;	// Up
		else if (keyCode == down)	controller1 |= 0x02;	// Down
		else if (keyCode == left)	controller1 |= 0x04;	// Left
		else if (keyCode == right)	controller1 |= 0x08;	// Right
		else if (keyCode == fire1)	controller1 |= 0x10;	// Fire 1
		else if (keyCode == fire2)	controller1 |= 0x20;	// Fire 2
		else if (keyCode == start)
		{
			if (setup.is_sms)
				setup.pause_button = false;	// Pause
			else
				ggstart	|= 0x80;				// Start
		}
	}

	public void keyTyped(KeyEvent evt){}
	public boolean isFocusTraversable() { return true; }

	/**
	 *	Get Status of Control Pad 1
	 *
	 *  @return		An Integer representing the state the control pad.
     */

	public int getController1()
	{
		return controller1;
	}

	/**
	 *	Get Status of Control Pad 2
	 *
	 *  @return		An Integer representing the state the control pad.
     */

	public int getController2()
	{
		if (!reset_button)
			return controller2;
		else
		{
			reset_button = false;
			return controller2 &~0x10;
		}
	}

	/**
	 *	Get Status of Game Gear Start Button
	 *
	 *  @return		An Integer representing the state of the button.
     */

	public int getGameGearStart()
	{
		return ggstart;
	}
	
	/**
	 *	Set SMS Reset Button
     */
	
	public void setReset()
	{
		reset_button = true;
	}

	/**
	 *	Redefine the key mapping.
	 *
	 *  @param newKeys	An Array of Key Codes
     */

	public void setKeys(int [] newKeys)
	{
		up    = newKeys[0];
		down  = newKeys[1];
		left  = newKeys[2];
		right = newKeys[3];
		fire1 = newKeys[4];
		fire2 = newKeys[5];
		start = newKeys[6];
	}

	/**
	 *	Get the current key mapping.
	 *
	 *  @return		An Array of Key Codes
     */

	public int[] getKeys()
	{
		int[] keys = {up, down, left, right, fire1, fire2, start};
		return keys;
	}
}