/**
 * Debug.java
 *
 * JavaGear Debug Frame
 *
 * @author Copyright (C) 2002 Chris White
 * @version 16th March 2002
 * @see "JavaGear Final Project Report"
 */

 /*
   This file is part of JavaGear.

   JavaGear is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   JavaGear is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with JavaGear; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.net.*; 		// URL Handling

public final class Debug extends JFrame
{
	private Z80 cpu;					// Pointer to CPU
	private Vdp vdp;					// Pointer to VDP
	private Screen screenpanel;	// Pointer to Java ScreenPanel

	/**
	 *	Debug Frame Constructor
	 *
	 *	@param z	Z80
	 *  @param v	Vdp
     *	@param s	ScreenPanel
     */

	public Debug(Z80 z, Vdp v, Screen s)
	{
		this.cpu = z;
		this.vdp = v;
		this.screenpanel = s;

		// Create 3 Main Tables
		JTable regTable = new JTable(regValues, regNames);
		JTable flagTable = new JTable(flagValues, flagNames);
		JTable debugTable = new JTable(debugValues, debugNames);

		// create 3 Split Panes
		JSplitPane regPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, new JScrollPane(regTable), new JScrollPane(flagTable));
		JSplitPane debugPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, new JScrollPane(debugTable), regPane);
		regPane.setResizeWeight(0.5);
		debugPane.setResizeWeight(0.75);

		final DebugInst debugins = new DebugInst(z, v, s, regTable, flagTable, debugTable);

		Container contentPane = getContentPane();

		// add panel for debug buttons
		JPanel panel = new JPanel();
		contentPane.add(panel, "South");

		// Setup Toolbar buttons
		Action inst1 = new AbstractAction("Execute 1 Instruction", new ImageIcon(getImage("images/debug1.gif")))
								{ 	public void actionPerformed(ActionEvent event)
									{
										debugins.execute(1);
									} };
		Action inst10 = new AbstractAction("Execute 10 Instructions", new ImageIcon(getImage("images/debug10.gif")))
								{ 	public void actionPerformed(ActionEvent event)
									{
										debugins.execute(10);
									} };
		Action inst100 = new AbstractAction("Execute 100 Instructions", new ImageIcon(getImage("images/debug100.gif")))
								{ 	public void actionPerformed(ActionEvent event)
									{
										debugins.execute(100);
									} };
		Action inst1000 = new AbstractAction("Execute 1000 Instructions", new ImageIcon(getImage("images/debug1000.gif")))
								{ 	public void actionPerformed(ActionEvent event)
									{
										debugins.execute(1000);
									} };
		Action inst10000 = new AbstractAction("Execute 10,000 Instructions", new ImageIcon(getImage("images/debug10k.gif")))
								{ 	public void actionPerformed(ActionEvent event)
									{
										debugins.execute(10000);
									} };
		Action inst100000 = new AbstractAction("Execute 100,000 Instructions", new ImageIcon(getImage("images/debug100k.gif")))
								{ 	public void actionPerformed(ActionEvent event)
									{
										debugins.execute(100000);
									} };

		// Create Tool Bar
		JToolBar bar = new JToolBar();
		bar.add(inst1); bar.add(inst10); bar.add(inst100); bar.add(inst1000); bar.add(inst10000); bar.add(inst100000);
		contentPane.add(bar, "South");

		// add main pane and show it all
		contentPane.add(debugPane, "Center");

		setTitle("Z80 Debugger");
		setSize(500,375);
		setResizable(false);
	}
	
	
	/**
	 * Get Image Within a JAR file 
	 * 
	 * @param filename	Filename to read
	 * 
	 * @returns	Image
	 */
	
	private Image getImage(String filename)
	{
		URL imageURL = KeyFrame.class.getResource(filename);
		return Toolkit.getDefaultToolkit().getImage(imageURL);	
	}

	// Arrays Holding Table Contents
	private String[] debugNames = {"PC", "OPCODE", "MNEMONIC"};
	private Object[][] debugValues = {
										{" ", " ", " "},
										{" ", " ", " "},
										{" ", " ", " "},
										{" ", " ", " "},
										{" ", " ", " "},
										{" ", " ", " "},
										{" ", " ", " "},
										{" ", " ", " "},
										{" ", " ", " "},
										{" ", " ", " "}
									};


	private String[] regNames = {"A", "F", "B", "C", "D", "E", "H", "L", "IX", "IY", "R", "SP"};
	private Object[][] regValues = {
									{" ", " ", " ", " ", " ", " ", " ", " ", " ", " ", " ", " "},
									{" ", " ", " ", " ", " ", " ", " ", " ", " ", " ", " ", " "}
								};


	private String[] flagNames = {"SIGN", "ZERO", "BIT 5", "HC", "BIT 3", "PV", "NEG", "CARRY"};
	private Object[][] flagValues = {
									{" ", " ", " ", " ", " ", " ", " ", " "},
									{" ", " ", " ", " ", " ", " ", " ", " "}
								};
}
