/**
 * DebugInst.java
 *
 * JavaGear Debugger
 *
 * @author Copyright (C) 2002 Chris White
 * @version 16th March 2002
 * @see "JavaGear Final Project Report"
 */

 /*
   This file is part of JavaGear.

   JavaGear is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   JavaGear is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with JavaGear; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import javax.swing.*; // For JTable Support

public final class DebugInst
{
	private Z80 cpu;							// Pointer to Z80 CPU
	private Vdp vdp;							// Pointer to VDP
	private Screen screenpanel;			// Pointer to Java Screenpanel
	private JTable regTable;					// Pointer to Registers Table
	private JTable flagTable;					// Pointer to Flag Table
	private JTable debugTable;					// Pointer to Debug Table

	private int lineno;							// Current Line Number

	/**
	 *	EmulateLoop Constructor
	 *
	 *	@param z		Z80
	 *  @param v		Vdp
     *	@param s		ScreenPanel
     *	@param regt		Register Table
     *	@param flagt	Flag Table
     *	@param debugt	Debug Table
     */

	public DebugInst(Z80 z, Vdp v, Screen s, JTable regt, JTable flagt, JTable debugt)
	{
		lineno = 0;
		cpu = z;
		vdp = v;
		screenpanel = s;
		regTable = regt;
		flagTable = flagt;
		debugTable = debugt;
	}


	/**
	 *	Execute a specific number of Z80 Instructions
	 *
	 *	@param count	Number of Instructions to Execute
     */

	public void execute(int count)
	{
		for (int x=count; x > 0; x--)
		{
			// If this is the last instruction
			if (x == 1)
			{
				// Update Last column of tables
				updateRegisters(1);
				updateFlags(1);
				updateDebug();
			}
			else if (x <= 10) updateDebug(); // Necessary to get correct previous instructions

			// If cycle count has been reached
			if(cpu.debug(227))
			{
				vdp.interrupts(lineno);
				vdp.drawLine(lineno);
				lineno++;					   // Increment line number

				if (lineno == 261)
				{
					lineno = 0; // Reset line number
					screenpanel.refresh();
				}
			}
		}

		// Update Column 0
		updateRegisters(0);
		updateFlags(0);
	}

	// Update Register Table
	private void updateRegisters(int row)
	{
		regTable.setValueAt(Integer.toHexString(cpu.getA()), row, 0);
		regTable.setValueAt(Integer.toHexString(cpu.getF()), row, 1);
		regTable.setValueAt(Integer.toHexString(cpu.getB()), row, 2);
		regTable.setValueAt(Integer.toHexString(cpu.getC()), row, 3);
		regTable.setValueAt(Integer.toHexString(cpu.getD()), row, 4);
		regTable.setValueAt(Integer.toHexString(cpu.getE()), row, 5);
		regTable.setValueAt(Integer.toHexString(cpu.getH()), row, 6);
		regTable.setValueAt(Integer.toHexString(cpu.getL()), row, 7);
		regTable.setValueAt(Integer.toHexString(cpu.getIX()), row, 8);
		regTable.setValueAt(Integer.toHexString(cpu.getIY()), row, 9);
		regTable.setValueAt(Integer.toHexString(cpu.getR()), row, 10);
		regTable.setValueAt(Integer.toHexString(cpu.getSP()), row, 11);
	}

	// Update Flag Table
	private void updateFlags(int row)
	{
		flagTable.setValueAt(new Boolean(cpu.getSign()), row, 0);
		flagTable.setValueAt(new Boolean(cpu.getZero()), row, 1);
		flagTable.setValueAt(new Boolean(cpu.getBit5()), row, 2);
		flagTable.setValueAt(new Boolean(cpu.getHc()), row, 3);
		flagTable.setValueAt(new Boolean(cpu.getBit3()), row, 4);
		flagTable.setValueAt(new Boolean(cpu.getParity()), row, 5);
		flagTable.setValueAt(new Boolean(cpu.getNegative()), row, 6);
		flagTable.setValueAt(new Boolean(cpu.getCarry()), row, 7);
	}

	// Update Debug Table
	private void updateDebug()
	{
		scrollDebug(0); scrollDebug(1); scrollDebug(2);
		debugTable.setValueAt(new String(cpu.getMnu()), 9, 2);
		debugTable.setValueAt("0x"+Integer.toHexString(cpu.getPC()), 9, 0);
		debugTable.setValueAt(new String(cpu.getOp()), 9, 1);
	}

	// Scroll the debug table upwards
	private void scrollDebug(int column)
	{
		for (int x = 0; x < 9; x++)
			debugTable.setValueAt(debugTable.getValueAt(x+1,column), x, column);
	}
}