/**
 * KeyFrame.java
 *
 * Create the Frame to Define Controller Configuration
 *
 * @author Copyright (C) 2002 Chris White
 * @version 16th March 2002
 * @see "JavaGear Final Project Report"
 */

 /*
    This file is part of JavaGear.

    JavaGear is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    JavaGear is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with JavaGear; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public final class KeyFrame extends JFrame
	implements ActionListener
{
	private Controllers controllers;		// Pointer to Controller Emulation
	private KeyConfig keyConfig;			// Pointer to keyConfig Panel
	private JPanel buttons;
	private JButton ok;
	private JButton cancel;


	/**
	 *	KeyFrame Constructor.
	 *
     *  @param c	Controllers
     */

	public KeyFrame(Controllers c)
	{
		this.controllers = c;
		setTitle("Key Configuration");

		// Get screen sizes
		Toolkit tk = Toolkit.getDefaultToolkit();
		Dimension d = tk.getScreenSize();
		// Set Window in middle
		setLocation(d.width / 4, d.height / 4);

		keyConfig = new KeyConfig();
		getContentPane().add(keyConfig);

		keyConfig.setKeys(controllers.getKeys());

		addWindowListener(new WindowAdapter()
		{
			public void windowClosing(WindowEvent e)
			{
				keyConfig.setKeys(controllers.getKeys()); // reload original settings
			 	hide();
			}
		});

		buttons = new JPanel(new GridLayout(1,2));

		ok = new JButton("OK");
		cancel = new JButton("Cancel");
		buttons.add(ok);
		buttons.add(cancel);

		getContentPane().add(buttons, "South");

		ok.addActionListener(this);
		cancel.addActionListener(this);

		setResizable(false);
		pack();
	}

	public void actionPerformed(ActionEvent evt)
	{
		if (evt.getSource() == ok)
		{
			controllers.setKeys(keyConfig.getKeys());
			this.hide();
		}
		else if (evt.getSource() == cancel)
		{
			keyConfig.setKeys(controllers.getKeys()); // reload original settings
			this.hide();
		}
	}

	public void paint(Graphics g)
	{
		keyConfig.refresh();
		buttons.repaint();
	}

}

