/**
 * Flag.java
 *
 * Zilog Z80 CPU Refresh Register Emulation
 *
 * @author Copyright (C) 2002-2003 Chris White
 * @version 18th January 2003
 * @see "JavaGear Final Project Report"
 */

 /*
    This file is part of JavaGear.

    JavaGear is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    JavaGear is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with JavaGear; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

public final class Refresh
{
	private int reg;
	private int temp;

	/**
	 *	Refresh Register Constructor.
     */

	public Refresh()
	{
		reg = 0;
	}

	/**
	 *  Clear Register
	 */

	public void clear()
	{
		reg = 0;
	}

	/**
	 *	Get Value of Refresh Register
	 *
	 *	@return				Value Stored in Flag Register
     */

	public int get()
	{
		return reg;
	}

	/**
	 *	Set Refresh Register
	 *
	 *	@param r			Value to set Register (0 - 0xFF)
     */

	public void set(int r)
	{
		reg = r&0xFF;
	}


	/**
	 * 	Increment Refresh Register
	 */

	public void inc()
	{
		// Increment lower 6 bits
		// Preserve original 'bit 7'
		reg = (reg&0x80) | ((reg+1)&0x7F);
	}
}