/**
 * Setup.java
 *
 * Global System Settings
 *
 * @author Copyright (C) 2002 Chris White
 * @version 17th March 2002
 * @see "JavaGear Final Project Report"
 */

 /*
    This file is part of JavaGear.

    JavaGear is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    JavaGear is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with JavaGear; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

public final class Setup
{
	public static final int SMS_WIDTH  = 256;			/** SMS Screen Width */
	public static final int SMS_HEIGHT = 192;			/** SMS Screen Height */

	public static final int GG_WIDTH   = 160;			/** GG Screen Width */
	public static final int GG_HEIGHT  = 144;			/** GG Screen Height */

	public static final int GG_X_OFFSET= 48;				/** GG Window Starts Here (x) */
	public static final int GG_Y_OFFSET= 24;				/** GG Window Starts Here (y) */

	public static boolean is_sms = true;					/** SMS Mode */
	public static boolean is_gg  = false;				/** GG Mode */

	public static int h_start = 0;						/** Horizontal Viewport Start */
	public static int h_end   = SMS_WIDTH;				/** Horizontal Viewport End */

	public static int v_start = 0;						/** Vertical Viewport Start */
	public static int v_end   = SMS_HEIGHT;				/** Vertical Viewport End */

	public static int h_start_scaled = 0;					/** Horizontal Viewport Start (Scaled Mode) */
	public static int v_start_scaled = 0;					/** Vertical Viewport Start (Scaled Mode) */

	public static int h_end_scaled = SMS_WIDTH;			/** Horizontal Viewport End (Scaled Mode) */
	public static int v_end_scaled = SMS_HEIGHT;			/** Vertical Viewport End (Scaled Mode) */

	public static int h_end_scaled_gg = 0;				/** Horizontal Viewport End (Scaled Game Gear Mode) */
	public static int v_end_scaled_gg = 0;				/** Vertical Viewport End (Scaled Game Gear Mode) */
	
	public static int scale;
	
	public static boolean pause_button;			// SMS Pause Button


	/**
	 *	Setup. Constructor.
	 *
	 *  Default to Screen Size 1
     */

	public Setup()
	{
		scale = 1;
		setScreenSize(scale);
	}


	/**
	 *	Set SMS Mode
     */

	public void setSMS()
	{
		is_sms = true;
		is_gg  = false;

		// SMS uses 256x192 window (32x28)
		h_start = 0;
		h_end   = SMS_WIDTH;

		v_start = 0;
		v_end   = SMS_HEIGHT;
	}

	/**
	 *	Set GG Mode
     */

	public void setGG()
	{
		is_gg  = true;
		is_sms = false;

		//h_start = 40; // should be 48 ie 6, but due to wierd hscroll code has to be 8 less.
		h_start = 40;
		h_end   = GG_WIDTH + GG_X_OFFSET;

		v_start = GG_Y_OFFSET;
		v_end   = GG_HEIGHT + GG_Y_OFFSET;
	}

	/**
	 *	Set Screen Size
	 *
	 *	@param s	Screen Size will be (normal * s)
     */

	public void setScreenSize(int s)
	{
		scale = s;
		h_end_scaled = SMS_WIDTH*scale;
		v_end_scaled = SMS_HEIGHT*scale;

		h_start_scaled = GG_X_OFFSET * scale;
		v_start_scaled = GG_Y_OFFSET * scale;

		h_end_scaled_gg = (GG_WIDTH + GG_X_OFFSET)*scale;
		v_end_scaled_gg = (GG_HEIGHT + GG_Y_OFFSET)*scale;
	}
}