/**
 * VdpDebugFrame.java
 *
 * Extract Debug Information from VDP.
 *
 * @author Copyright (C) 2002 Chris White
 * @version 17th March 2002
 * @see "JavaGear Final Project Report"
 */

 /*
   This file is part of JavaGear.

   JavaGear is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   JavaGear is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with JavaGear; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

public final class VdpDebugFrame extends JFrame
	implements ActionListener
{
	// Register Labels
	private Label reg0 = new Label("");
	private Label reg1 = new Label("");
	private Label reg2 = new Label("");
	private Label reg3 = new Label("");
	private Label reg4 = new Label("");
	private Label reg5 = new Label("");
	private Label reg6 = new Label("");
	private Label reg7 = new Label("");
	private Label reg8 = new Label("");
	private Label reg9 = new Label("");
	private Label reg10= new Label("");

	// Interrupt Labels
	private Label current_line 		= new Label("");
	private Label line_counter 		= new Label("");
	private Label line_ints    		= new Label("");
	private Label line_ints_pending = new Label("");
	private Label frame_ints    	= new Label("");
	private Label frame_ints_pending= new Label("");
	private Label z80_int_line	    = new Label("");

	// Background Labels
	private Label bgt 				= new Label("");
	private Label h_scroll	 		= new Label("");
	private Label v_scroll    		= new Label("");
	private Label lock_rows			= new Label("");
	private Label lock_columns   	= new Label("");

	// Sprite Labels
	private Label sat 				= new Label("");
	private Label high_sprites		= new Label("");
	private Label zoom_sprites   	= new Label("");
	private Label shift_left		= new Label("");
	private Label sprite_overflow   = new Label("");

	private JPanel buttons;
	private JButton refresh;
	private JButton cancel;

	private Vdp vdp;				// Pointer to VDP

	public VdpDebugFrame(Vdp v)
	{
		setTitle("VDP Debug Info");

		this.vdp = v;

		Border raised = BorderFactory.createRaisedBevelBorder();
		Border reg_border = BorderFactory.createTitledBorder(raised, "Registers");
		Border int_border = BorderFactory.createTitledBorder(raised, "Interrupts");
		Border bg_border = BorderFactory.createTitledBorder(raised, "Background Layer");
		Border sprite_border = BorderFactory.createTitledBorder(raised, "Sprite Layer");

		JPanel reg_panel = new JPanel(new GridLayout(6,4)); // h, v, hgap, vgap
		reg_panel.setBorder(reg_border);
		reg_panel.add(new Label("Reg 0 : "));
		reg_panel.add(reg0);
		reg_panel.add(new Label("Reg 1 : "));
		reg_panel.add(reg1);
		reg_panel.add(new Label("Reg 2 : "));
		reg_panel.add(reg2);
		reg_panel.add(new Label("Reg 3 : "));
		reg_panel.add(reg3);
		reg_panel.add(new Label("Reg 4 : "));
		reg_panel.add(reg4);
		reg_panel.add(new Label("Reg 5 : "));
		reg_panel.add(reg5);
		reg_panel.add(new Label("Reg 6 : "));
		reg_panel.add(reg6);
		reg_panel.add(new Label("Reg 7 : "));
		reg_panel.add(reg7);
		reg_panel.add(new Label("Reg 8 : "));
		reg_panel.add(reg8);
		reg_panel.add(new Label("Reg 9 : "));
		reg_panel.add(reg9);
		reg_panel.add(new Label("Reg 10: "));
		reg_panel.add(reg10);

		JPanel int_panel = new JPanel(new GridLayout(7,2));
		int_panel.setBorder(int_border);
		int_panel.add(new Label("Current Line : "));
		int_panel.add(current_line);
		int_panel.add(new Label("Line Counter : "));
		int_panel.add(line_counter);
		int_panel.add(new Label("Line Interrupts : "));
		int_panel.add(line_ints);
		int_panel.add(new Label("Line Interrupt Pending  : "));
		int_panel.add(line_ints_pending);
		int_panel.add(new Label("Frame Interrupts : "));
		int_panel.add(frame_ints);
		int_panel.add(new Label("Frame Interrupt Pending : "));
		int_panel.add(frame_ints_pending);
		int_panel.add(new Label("Z80 Interrupt Line : "));
		int_panel.add(z80_int_line);

		JPanel bg_panel = new JPanel(new GridLayout(5,2));
		bg_panel.setBorder(bg_border);
		bg_panel.add(new Label("Background Table : "));
		bg_panel.add(bgt);
		bg_panel.add(new Label("Horizontal Scroll : "));
		bg_panel.add(h_scroll);
		bg_panel.add(new Label("Vertical Scroll : "));
		bg_panel.add(v_scroll);
		bg_panel.add(new Label("Lock Top Two Rows : "));
		bg_panel.add(lock_rows);
		bg_panel.add(new Label("Lock Right Eight Columns : "));
		bg_panel.add(lock_columns);

		JPanel sprite_panel = new JPanel(new GridLayout(5,2));
		sprite_panel.setBorder(sprite_border);
		sprite_panel.add(new Label("Sprite Attribute Table : "));
		sprite_panel.add(sat);
		sprite_panel.add(new Label("8x16 Sprites : "));
		sprite_panel.add(high_sprites);
		sprite_panel.add(new Label("Zoom Sprites : "));
		sprite_panel.add(zoom_sprites);
		sprite_panel.add(new Label("Shift 8 Pixels To Left : "));
		sprite_panel.add(shift_left);
		sprite_panel.add(new Label("Sprite Overflow : "));
		sprite_panel.add(sprite_overflow);

		JPanel info_panel = new JPanel(new GridLayout(2,2));
		info_panel.add(bg_panel);
		info_panel.add(sprite_panel);
		info_panel.add(reg_panel);
		info_panel.add(int_panel);

		buttons = new JPanel(new GridLayout(1,2));
		refresh = new JButton("Refresh");
		cancel = new JButton("Cancel");
		buttons.add(refresh);
		buttons.add(cancel);

		getContentPane().add(info_panel);
		getContentPane().add(buttons, "South");

		refresh.addActionListener(this);
		cancel.addActionListener(this);

		setResizable(false);
		pack(); pack();
		refresh();
	}

	public void actionPerformed(ActionEvent evt)
	{
		if (evt.getSource() == refresh)
			this.refresh();
		else if (evt.getSource() == cancel)
			this.hide();
	}

	public void refresh()
	{
		// Set Registers Panel
		reg0.setText(Integer.toBinaryString(vdp.getReg(0)));
		reg1.setText(Integer.toBinaryString(vdp.getReg(1)));
		reg2.setText(Integer.toBinaryString(vdp.getReg(2)));
		reg3.setText(Integer.toBinaryString(vdp.getReg(3)));
		reg4.setText(Integer.toBinaryString(vdp.getReg(4)));
		reg5.setText(Integer.toBinaryString(vdp.getReg(5)));
		reg6.setText(Integer.toBinaryString(vdp.getReg(6)));
		reg7.setText(Integer.toBinaryString(vdp.getReg(7)));
		reg8.setText(Integer.toBinaryString(vdp.getReg(8)));
		reg9.setText(Integer.toBinaryString(vdp.getReg(9)));
		reg10.setText(Integer.toBinaryString(vdp.getReg(10)));

		// Set Interrupt Panel
		current_line.setText(Integer.toString(vdp.getLineNo()));
		line_counter.setText(Integer.toString(vdp.getCounter()) + " / " +
							 Integer.toString(vdp.getReg(10)));

		if ((vdp.getReg(0) & 0x10) == 0x10)
			line_ints.setText("Enabled");
		else
			line_ints.setText("Disabled");

		line_ints_pending.setText(new Boolean(vdp.getLineIntPending()).toString());

		if ((vdp.getReg(1) & 0x20) == 0x20)
			frame_ints.setText("Enabled");
		else
			frame_ints.setText("Disabled");

		frame_ints_pending.setText(new Boolean(vdp.getFrameIntPending()).toString());
		z80_int_line.setText(new Boolean(vdp.getIrqStatus()).toString());

		// Set Background Panel
		bgt.setText("0x"+Integer.toHexString( (vdp.getReg(2) &0x0f &~0x01)<<10) );
		h_scroll.setText(Integer.toString(vdp.getReg(8)));
		v_scroll.setText(Integer.toString(vdp.getReg(9)));

		if ((vdp.getReg(0) & 0x40) == 0x40)
			lock_rows.setText("Enabled");
		else
			lock_rows.setText("Disabled");

		if ((vdp.getReg(0) & 0x80) == 0x80)
			lock_columns.setText("Enabled");
		else
			lock_columns.setText("Disabled");

		// Set Sprite Panel
		sat.setText("0x"+Integer.toHexString((vdp.getReg(5) &~ 0x01 &~ 0x80)<<7) );
		if ((vdp.getReg(1) & 0x02) == 0x02)
			high_sprites.setText("Enabled");
		else
			high_sprites.setText("Disabled");

		if ((vdp.getReg(1) & 0x01) == 0x01)
			zoom_sprites.setText("Enabled");
		else
			zoom_sprites.setText("Disabled");

		if ((vdp.getReg(0) & 0x08) == 0x08)
			shift_left.setText("Enabled");
		else
			shift_left.setText("Disabled");

		sprite_overflow.setText(new Boolean(vdp.getSpriteOverflow()).toString());
	}
}
